/*
  ;;;;
  ;;;; ecl-readline.lisp
  ;;;;
  ;;;; Copyright (C) 2008 Jason Aquilina <jfa7600@yahoo.com.au>
  ;;;;
  ;;;; This program is free software; you can redistribute it and/or
  ;;;; modify it under the terms of the GNU General Public License
  ;;;; as published by the Free Software Foundation; either version 2
  ;;;; of the License, or (at your option) any later version.
  ;;;;
  ;;;; This program is distributed in the hope that it will be useful,
  ;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
  ;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  ;;;; GNU General Public License for more details.
  ;;;;
  ;;;; You should have received a copy of the GNU General Public License
  ;;;; along with this program; if not, write to the Free Software
  ;;;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  ;;;;

  ;;;;
  ;;;; Readline support for ECL
  ;;;;
  ;;;; Ideas and code stolen from:
  ;;;;    CL-READLINE by Nikodemus Siivola <nikodemus@random-state.net>
  ;;;;    SB-READLINE by Miles Egan <miles@caddr.com>
  ;;;;
*/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <readline/readline.h>

static char** g_completions = NULL;
static int g_index = 0;

void add_completion_entry(char* text)
{
  g_completions[g_index++] = strdup(text);
}


void allocate_completions(int length)
{
  if (g_completions) {
    char** p = g_completions;
    while (*p)
      free(*p++);

    free(g_completions);
  }

  g_index = 0;
  g_completions = malloc(length * sizeof(char*) + 1);
  g_completions[length] = (char*)NULL;
}


char* symbol_completer(const char* str, int target)
{
  static char** ptr  = NULL;   /* pointer into completions array */
  static int length = 0;       /* cached length of str         */

  if (target == 0) {
    ptr = g_completions;
    length = strlen(str);

    while (*ptr && strncmp(str, *ptr, length) != 0)
      ptr++;

    return *ptr ? strdup(*ptr) : (char*)NULL;
  }

  if (*ptr && strncmp(str, *ptr, length) == 0)
      return strdup(*ptr++);

  return (char*)NULL;
}


void use_symbol_completer()
{
  rl_completion_entry_function = symbol_completer;
}

/* End of File */
